@@ -2,14 +2,17 @@
 Functions for dealing with markup text
 """
 
+from __future__ import annotations
+
 import re
+from collections.abc import Iterable
 from html.entities import name2codepoint
-from typing import Iterable, Match, AnyStr, Optional, Pattern, Tuple, Union
+from re import Match, Pattern
 from urllib.parse import urljoin
 
-from w3lib.util import to_unicode
-from w3lib.url import safe_url_string
 from w3lib._types import StrOrBytes
+from w3lib.url import safe_url_string
+from w3lib.util import to_unicode
 
 _ent_re = re.compile(
     r"&((?P<named>[a-z\d]+)|#(?P<dec>\d+)|#x(?P<hex>[a-f\d]+))(?P<semicolon>;?)",
@@ -34,7 +37,7 @@ HTML5_WHITESPACE = " \t\n\r\x0c"
 
 
 def replace_entities(
-    text: AnyStr,
+    text: StrOrBytes,
     keep: Iterable[str] = (),
     remove_illegal: bool = True,
     encoding: str = "utf-8",
@@ -77,10 +80,9 @@ def replace_entities(
             entity_name = groups["named"]
             if entity_name.lower() in keep:
                 return m.group(0)
-            else:
-                number = name2codepoint.get(entity_name) or name2codepoint.get(
-                    entity_name.lower()
-                )
+            number = name2codepoint.get(entity_name) or name2codepoint.get(
+                entity_name.lower()
+            )
         if number is not None:
             # Numeric character references in the 80-9F range are typically
             # interpreted by browsers as representing the characters mapped
@@ -89,8 +91,7 @@ def replace_entities(
             try:
                 if 0x80 <= number <= 0x9F:
                     return bytes((number,)).decode("cp1252")
-                else:
-                    return chr(number)
+                return chr(number)
             except (ValueError, OverflowError):
                 pass
 
@@ -99,11 +100,11 @@ def replace_entities(
     return _ent_re.sub(convert_entity, to_unicode(text, encoding))
 
 
-def has_entities(text: AnyStr, encoding: Optional[str] = None) -> bool:
+def has_entities(text: StrOrBytes, encoding: str | None = None) -> bool:
     return bool(_ent_re.search(to_unicode(text, encoding)))
 
 
-def replace_tags(text: AnyStr, token: str = "", encoding: Optional[str] = None) -> str:
+def replace_tags(text: StrOrBytes, token: str = "", encoding: str | None = None) -> str:
     """Replace all markup tags found in the given `text` by the given token.
     By default `token` is an empty string so it just removes all tags.
 
@@ -129,7 +130,7 @@ def replace_tags(text: AnyStr, token: str = "", encoding: Optional[str] = None)
 _REMOVECOMMENTS_RE = re.compile("<!--.*?(?:-->|$)", re.DOTALL)
 
 
-def remove_comments(text: AnyStr, encoding: Optional[str] = None) -> str:
+def remove_comments(text: StrOrBytes, encoding: str | None = None) -> str:
     """Remove HTML Comments.
 
     >>> import w3lib.html
@@ -144,10 +145,10 @@ def remove_comments(text: AnyStr, encoding: Optional[str] = None) -> str:
 
 
 def remove_tags(
-    text: AnyStr,
+    text: StrOrBytes,
     which_ones: Iterable[str] = (),
     keep: Iterable[str] = (),
-    encoding: Optional[str] = None,
+    encoding: str | None = None,
 ) -> str:
     """Remove HTML Tags only.
 
@@ -202,8 +203,7 @@ def remove_tags(
         tag = tag.lower()
         if which_ones:
             return tag in which_ones
-        else:
-            return tag not in keep
+        return tag not in keep
 
     def remove_tag(m: Match[str]) -> str:
         tag = m.group(1)
@@ -216,7 +216,7 @@ def remove_tags(
 
 
 def remove_tags_with_content(
-    text: AnyStr, which_ones: Iterable[str] = (), encoding: Optional[str] = None
+    text: StrOrBytes, which_ones: Iterable[str] = (), encoding: str | None = None
 ) -> str:
     """Remove tags and their content.
 
@@ -240,10 +240,10 @@ def remove_tags_with_content(
 
 
 def replace_escape_chars(
-    text: AnyStr,
+    text: StrOrBytes,
     which_ones: Iterable[str] = ("\n", "\t", "\r"),
     replace_by: StrOrBytes = "",
-    encoding: Optional[str] = None,
+    encoding: str | None = None,
 ) -> str:
     """Remove escape characters.
 
@@ -262,10 +262,10 @@ def replace_escape_chars(
 
 
 def unquote_markup(
-    text: AnyStr,
+    text: StrOrBytes,
     keep: Iterable[str] = (),
     remove_illegal: bool = True,
-    encoding: Optional[str] = None,
+    encoding: str | None = None,
 ) -> str:
     """
     This function receives markup as a text (always a unicode string or
@@ -278,9 +278,7 @@ def unquote_markup(
 
     """
 
-    def _get_fragments(
-        txt: str, pattern: Pattern[str]
-    ) -> Iterable[Union[str, Match[str]]]:
+    def _get_fragments(txt: str, pattern: Pattern[str]) -> Iterable[str | Match[str]]:
         offset = 0
         for match in pattern.finditer(txt):
             match_s, match_e = match.span(1)
@@ -304,7 +302,7 @@ def unquote_markup(
 
 
 def get_base_url(
-    text: AnyStr, baseurl: StrOrBytes = "", encoding: str = "utf-8"
+    text: StrOrBytes, baseurl: StrOrBytes = "", encoding: str = "utf-8"
 ) -> str:
     """Return the base url if declared in the given HTML `text`,
     relative to the given base url.
@@ -314,22 +312,20 @@ def get_base_url(
     """
 
     utext: str = remove_comments(text, encoding=encoding)
-    m = _baseurl_re.search(utext)
-    if m:
+    if m := _baseurl_re.search(utext):
         return urljoin(
             safe_url_string(baseurl), safe_url_string(m.group(1), encoding=encoding)
         )
-    else:
-        return safe_url_string(baseurl)
+    return safe_url_string(baseurl)
 
 
 def get_meta_refresh(
-    text: AnyStr,
+    text: StrOrBytes,
     baseurl: str = "",
     encoding: str = "utf-8",
     ignore_tags: Iterable[str] = ("script", "noscript"),
-) -> Tuple[Optional[float], Optional[str]]:
-    """Return  the http-equiv parameter of the HTML meta element from the given
+) -> tuple[None, None] | tuple[float, str]:
+    """Return the http-equiv parameter of the HTML meta element from the given
     HTML text and return a tuple ``(interval, url)`` where interval is an integer
     containing the delay in seconds (or zero if not present) and url is a
     string with the absolute url to redirect.
@@ -345,14 +341,12 @@ def get_meta_refresh(
         raise
     utext = remove_tags_with_content(utext, ignore_tags)
     utext = remove_comments(replace_entities(utext))
-    m = _meta_refresh_re.search(utext) or _meta_refresh_re2.search(utext)
-    if m:
+    if m := _meta_refresh_re.search(utext) or _meta_refresh_re2.search(utext):
         interval = float(m.group("int"))
         url = safe_url_string(m.group("url").strip(" \"'"), encoding)
         url = urljoin(baseurl, url)
         return interval, url
-    else:
-        return None, None
+    return None, None
 
 
 def strip_html5_whitespace(text: str) -> str:
